﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Proteus;
using System.IO;
namespace Spark.Installer
{

    public class InstallerFile
    {
        public string InstallerPath;
        public string LocalPath;
        public InstallerFile() { }
        public InstallerFile(string local, string install)
        {
            InstallerPath = install;
            LocalPath = local;
        }
    }

    public class InstallerConfig
    {
        private string _strCurrentSection = InstallerSection.None;

        private List<InstallerFile> _lstFiles = new List<InstallerFile>();// All files. This is the final list of files the installer will package.

        private List<InstallerFile> _lstIncludeFiles = new List<InstallerFile>();
        private List<InstallerFile> _lstExcludeFiles = new List<InstallerFile>();
        private List<InstallerFile> _lstIncludeDirs = new List<InstallerFile>();
        private List<InstallerFile> _lstExcludeDirs = new List<InstallerFile>();

        private InstallOptions _objInstallOptions = new InstallOptions();

        private List<string> _lstExcludeExtensions = new List<string>();
        private string _strConfigFilePath = string.Empty;

        public int NumExcludedFiles = 0;
        public int NumExcludedDirectories = 0;
        public const char OptionArgumentDelimiter = '\u0006';// unicode char 6.  this separates InstallOption(s)

        public InstallOptions Options { get { return _objInstallOptions; } private set { _objInstallOptions = value;  } }

        public List<InstallerFile> GetFiles() { return _lstFiles; }
        public InstallerConfig(string path)
        {
            //Make sure the path is rooted
            _strConfigFilePath = path;

            if (!System.IO.Path.IsPathRooted(_strConfigFilePath))
                _strConfigFilePath = System.IO.Path.GetFullPath(_strConfigFilePath);
        }
        public byte[] Serialize()
        {
            return _objInstallOptions.Serialize();
        }
        private bool IsDirectoryAndFormat(ref string path)
        {
            FileAttributes attr;
            attr = FileAttributes.Normal;

            // Append location of config file to path.
            if (!System.IO.Path.IsPathRooted(path))
            {
                string strConfigPath = System.IO.Path.GetDirectoryName(_strConfigFilePath);
                path = System.IO.Path.Combine(strConfigPath, path);
            }

            try
            {
                attr = File.GetAttributes(path);
            }
            catch (DirectoryNotFoundException)
            {
                Globals.Throw("Directory " + path + " doesn't exist. While compiling file list. (01875912)");
            }
            catch (FileNotFoundException)
            {
                Globals.Throw("File " + path + " doesn't exist. While compiling file list. (01899625)");
            }

            if ((attr & FileAttributes.Directory) == FileAttributes.Directory)
                return true;
            return false;
        }
        private void ThrowInvalidNumberOfParameters(string pname, int line)
        {
            Globals.Throw("Invalid number of parameters to argument " + pname + " at line " + line.ToString());
        }

        #region Config
        public void Load()
        {
            string[] lines = System.IO.File.ReadAllLines(_strConfigFilePath);
            for (int iLine = 0; iLine < lines.Length; iLine++)
            {
                ParseCommand(lines[iLine], iLine);
            }
        }
        private bool IsComment(string line)
        {
            string st = line.TrimStart();
            if (st[0] == '#')
                return true;
            return false;
        }
        private bool isWs(char c)
        {
            return (c == ' ' || c == '\t' || c == '\r' || c == '\n');
        }
        private string[] ParseLine(string line)
        {
            List<string> tokens = new List<string>();
            string curToken = string.Empty;
            bool instrDq = false;
            bool instrSq = false;
            foreach (char c in line.ToCharArray())
            {
                if ( isWs(c) && !instrDq && !instrSq && curToken.Length > 0)
                {
                    //Whitespace
                    tokens.Add(curToken);
                    curToken = string.Empty;
                }
                else if ((c == '\'') && !instrDq)
                {
                    //Single quoted string
                    curToken += c;
                    instrSq = !instrSq;
                }
                else if ((c == '"') && !instrSq)
                {
                    //Double Quoted string
                    curToken += c;
                    instrDq = !instrDq;
                }
                else if ((c == '#') && !instrDq && !instrSq)
                {
                    //Line comment
                    break;
                }
                else if (c == '\n')
                {
                    //End of line
                    break;
                }
                else
                {
                    if( !isWs(c) || instrDq || instrSq ) // Ignore delimiting whitespace
                        curToken += c;
                }

            }
            if (curToken.Length > 0)
                tokens.Add(curToken);
            return tokens.ToArray();
            //line.Split(new char[] { ' ', '\t' }, StringSplitOptions.RemoveEmptyEntries);
        }
        private void ParseCommand(string line, int iLine)
        {
            line = line.Trim();
            if (string.IsNullOrEmpty(line))
                return;
            if (IsComment(line))
                return;

            string[] args = ParseLine(line);

            if (args.Length == 0)
                return;

            for (int i = 0; i < args.Length; i++)
                args[i] = args[i].Trim();

            List<string> lstInstallOptionKeys = new List<string> {
                InstallOption.DefaultDirectory          ,
                InstallOption.DisplayName               ,
                //InstallOption.CreateDesktopShortcut     ,
                //InstallOption.CreateStartMenuFolder     ,
                InstallOption.InstallLocation           ,
                InstallOption.Publisher                 ,
                InstallOption.DisplayIcon               ,
                InstallOption.ApplicationVersion        ,
                InstallOption.DisplayVersion            ,
                InstallOption.URLInfoAbout              ,
                InstallOption.Contact                   ,
                InstallOption.AppGuid                   ,
                InstallOption.ExecuteSync               ,
                InstallOption.DeleteFile                ,
                InstallOption.Shortcut                  ,
           };


            if (args[0].Equals("BuildSection"))
            {
                _strCurrentSection = InstallerSection.BuildSection;
            }
            else if (args[0].Equals("InstallSection"))
            {
                _strCurrentSection = InstallerSection.InstallSection;
            }
            else if (args[0].Equals("PreInstallSection"))
            {
                _strCurrentSection = InstallerSection.PreInstallSection;
            }
            else if (args[0].Equals("PostInstallSection"))
            {
                _strCurrentSection = InstallerSection.PostInstallSection;
            }
            else if (args[0].Equals("PreUninstallSection"))
            {
                _strCurrentSection = InstallerSection.PreUninstallSection;
            }
            else if (args[0].Equals("PostUninstallSection"))
            {
                _strCurrentSection = InstallerSection.PostUninstallSection;
            }
            else if (args[0].Equals("UninstallSection"))
            {
                _strCurrentSection = InstallerSection.UninstallSection;
            }
            else if (args[0].Equals("Include"))
            {
                ValidateSection(args[0], new List<string> { InstallerSection.BuildSection });

                if (args.Length != 3)
                    ThrowInvalidNumberOfParameters(args[0], iLine);
                string local = StringUtils.Dequote(args[1]);
                string install = StringUtils.Dequote(args[2]);

                InstallerFile newFile = new InstallerFile(local, install);

                if (IsDirectoryAndFormat(ref newFile.LocalPath))
                    _lstIncludeDirs.Add(newFile);
                else
                    _lstIncludeFiles.Add(newFile);
            }
            else if (args[0].Equals("Exclude"))
            {
                ValidateSection(args[0], new List<string> { InstallerSection.BuildSection });
                if (args.Length != 2)
                    ThrowInvalidNumberOfParameters(args[0], iLine);
                string local = StringUtils.Dequote(args[1]);

                InstallerFile newFile = new InstallerFile(local, "");

                if (IsDirectoryAndFormat(ref newFile.LocalPath))
                    _lstExcludeDirs.Add(newFile);
                else
                    _lstExcludeFiles.Add(newFile);
            }
            else if (args[0].Equals("ExcludeExt"))
            {
                ValidateSection(args[0], new List<string> { InstallerSection.BuildSection });
                if (args.Length != 2)
                    ThrowInvalidNumberOfParameters(args[0], iLine);
                string ext = StringUtils.Dequote(args[1]);
                ext = ext.ToLower();
                _lstExcludeExtensions.Add(ext);
            }
            //Option Directives
            else if (lstInstallOptionKeys.Contains(args[0]))
            {
                if (args[0].Equals(InstallOption.Shortcut))
                {
                    // 6 args for shortcuts (5 arguments 1 command)
                    if (args.Length != 6)
                        ThrowInvalidNumberOfParameters(args[0], iLine);
                }
                else if (args.Length != 2)
                    ThrowInvalidNumberOfParameters(args[0], iLine);

                InstallerSectionType st = InstallerSection.SectionStringToSectionType(_strCurrentSection);

                string strPipeDelimitedOptionString = "";
                string appender = "";
                for (int i = 1; i < args.Length; i++)
                {
                    strPipeDelimitedOptionString += appender + StringUtils.Dequote(args[i]);
                    appender = OptionArgumentDelimiter.ToString();
                }
                InstallOption newOpt = new InstallOption(args[0], strPipeDelimitedOptionString, st);
                _objInstallOptions.Options.Add(newOpt);
            }
            else
            {
                Globals.Logger.LogError("Unrecognized installer config command " + args[0], true);
            }
        }

        private void ValidateSection(string strOption, List<string> lstSections)
        {
            if (lstSections.Where(x => x.Equals(_strCurrentSection)).FirstOrDefault() == null)
                throw new Exception("Installer encountered invalid option '" + strOption + "' in section " + _strCurrentSection);
        }
        #endregion

        public void BuildFileList()
        {
            if (_lstIncludeFiles.Count == 0 && _lstIncludeDirs.Count == 0)
                Globals.Throw("There were no files or directories to include.  Please check your config file: " + _strConfigFilePath);

            foreach (InstallerFile instDir in _lstIncludeDirs)
            {
                ExpandDirectory(instDir.LocalPath, instDir);
            }
            foreach (InstallerFile instFile in _lstIncludeFiles)
            {
                if (!FileIsExcluded(instFile.LocalPath))
                    _lstFiles.Add(instFile);
            }
        }
        private bool DirectoryIsExcluded(string path)
        {
            int count;
            if (_lstExcludeDirs == null)
                return false;

            count = _lstExcludeDirs.Where(x => x.LocalPath.Equals(path)).Count();
            if (count > 0)
                return true;

            return false;
        }
        private bool FileIsExcluded(string path)
        {
            int count;
            if (_lstExcludeFiles == null)
                return false;

            count = _lstExcludeFiles.Where(x => x.LocalPath.Equals(path)).Count();
            if (count > 0)
                return true;

            string ext = System.IO.Path.GetExtension(path).ToLower();
            count = _lstExcludeExtensions.Where(x => x.Equals(ext)).Count();
            if (count > 0)
                return true;

            return false;
        }
        private void ExpandDirectory(string curDir, InstallerFile rootDir)
        {
            string baseDir = System.IO.Directory.GetCurrentDirectory();
            System.IO.Directory.SetCurrentDirectory(curDir);
            string[] files = System.IO.Directory.GetFiles(curDir);

            foreach (string file in files)
            {
                //Files must be absolute paths.
                if (!FileIsExcluded(file))
                {
                    string relPath;

                    relPath = file.Substring(rootDir.LocalPath.Length, file.Length - rootDir.LocalPath.Length);
                    relPath = StringUtils.CombinePath(rootDir.InstallerPath, relPath);
                    _lstFiles.Add(new InstallerFile(file, relPath));
                }
                else
                    NumExcludedFiles++;
            }

            string[] dirs = System.IO.Directory.GetDirectories(curDir);
            foreach (string dir in dirs)
            {
                if (!DirectoryIsExcluded(dir))
                    ExpandDirectory(dir, rootDir);
                else
                    NumExcludedDirectories++;
            }

            System.IO.Directory.SetCurrentDirectory(baseDir);
        }

    }
}
